//+------------------------------------------------------------------+
//|                                               premier stochastic |
//|                                                           mladen |
//|                                                                  |
//| Based on Lee Leibfarth article in TASC August 2008               |
//| article : "Premier Stochastic Oscillator."                       |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  5
#property indicator_color1   Green
#property indicator_color2   Green
#property indicator_color3   Red
#property indicator_color4   Red
#property indicator_color5   DimGray
#property indicator_width1   2
#property indicator_width3   2
#property indicator_width5   2
#property indicator_maximum  1
#property indicator_minimum -1
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern int    StochasticLength =  8;
extern int    StochasticMode   =  0;
extern int    SmoothLength     = 25;
extern double Level1           = 0.9;
extern double Level2           = 0.2;

//
//
//
//
//

double BufferUu[];
double BufferUd[];
double BufferDd[];
double BufferDu[];
double Buffer[];
double averages[][3];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,BufferUu); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,BufferUd); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,BufferDd); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,BufferDu); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,Buffer);
      SetLevelValue(0, Level1);
      SetLevelValue(1, Level2);
      SetLevelValue(2, 0.00);
      SetLevelValue(3,-Level2);
      SetLevelValue(4,-Level1);
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int start()
{
   int counted_bars = IndicatorCounted();
   int limit,i,r;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = Bars-counted_bars;
         if (ArrayRange(averages,0) != Bars) ArrayResize(averages,Bars);

   //
   //
   //
   //
   //

   double alpha = 2.0/(MathSqrt(SmoothLength)+1.0);
   for(i = limit, r=Bars-i-1; i >= 0 ; i--, r++)
   {
      averages[r][0] = 0.1*(iStochastic(NULL,0,StochasticLength,1,1,MODE_SMA,StochasticMode,MODE_MAIN,i)-50);
      averages[r][1] = averages[r-1][1]+alpha*(averages[r][0]-averages[r-1][1]);
      averages[r][2] = averages[r-1][2]+alpha*(averages[r][1]-averages[r-1][2]);

         double exp  = MathExp(averages[r][2]);
         
      //
      //
      //
      //
      //
      
      Buffer[i] = (exp-1)/(exp+1);
         BufferUu[i] = EMPTY_VALUE;
         BufferUd[i] = EMPTY_VALUE;
         BufferDd[i] = EMPTY_VALUE;
         BufferDu[i] = EMPTY_VALUE;
         if (Buffer[i] > 0)
            {
               if (Buffer[i] > Level1) { BufferUu[i] = Buffer[i]; continue; }
               if (Buffer[i] > Level2) { BufferUd[i] = Buffer[i]; continue; }
            }
         if (Buffer[i] < 0)
            {
               if (Buffer[i] < -Level1) { BufferDd[i] = Buffer[i]; continue;}
               if (Buffer[i] < -Level2) { BufferDu[i] = Buffer[i]; }
            }
   }

   //
   //
   //
   //
   //

   return(0);
}